/*
** Display a Zoner ZBR file preview bitmap under Microsoft Windows 3.1
**
** Author: James D. Murray <jdm@netcom.com>
** Date:   20 October 1995
**
** This is a minimal Windows application used to display the preview image
** stored in a Zoner 1.x .ZBR metafile. This app is by no means robust, and
** should be used as example code only.
**
** Supply the name of the file to view on the command line via the Program
** Manager's File/Run... dialog box, or on the DOS command line when starting
** up Windows. Exit this app using Close from the system menu or Alt-F4.
**
** For the Borland C++ compiler, compile from the DOS command line using:  
**      bcc -ml -W zbrprev.c
**
** For the Microsoft C++ compiler, compile from the DOS command line using:  
**      cl -AL zbrprev.c
*/
#define  STRICT
#include <windows.h>
#include <string.h>

#define ZBRFILEID       0x029a
#define ZBRFILEVER      0x0004
#define SIZEOFCOMMENT   100
#define SIZEOFPREVIEW   5264

char CommentData[SIZEOFCOMMENT];    /* Buffer to store ZBR file comment */
char PreviewData[SIZEOFPREVIEW];    /* Buffer to store ZBR preview data */

HINSTANCE hInst;        /* hInstance of application */
HWND      hWndMain;     /* hWnd of main window */
HDC       hDC;          /* Handle of the Main Window's device context */
RECT      Rect;         /* Main Window's device context rectangle */
LPSTR     lpszFileName; /* Name of the .ZBR file to open */

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpszCmdLine, int cmdShow);

LRESULT CALLBACK  _export ZbrPreviewWndProc(HWND hWnd, UINT message,
  WPARAM wParam, LPARAM lParam);

BOOL DisplayZbrFilePreview(void);
int ZBRReadHeader(LPSTR FileName);
int ZBRDrawPreview(HDC dc, RECT dest);


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpszCmdLine, int cmdShow)
{
    MSG        msg;
    WNDCLASS   wcZbrPreviewClass;

    /* Didn't give a file name on the command line */
    if (lpszCmdLine[0] == NULL)
        return(FALSE);
    lpszFileName = lpszCmdLine;

    /* Define the window class for this application */
    wcZbrPreviewClass.lpszClassName = lpszCmdLine;
    wcZbrPreviewClass.hInstance     = hInstance;
    wcZbrPreviewClass.lpfnWndProc   = ZbrPreviewWndProc;
    wcZbrPreviewClass.hCursor       = NULL;
    wcZbrPreviewClass.hIcon         = NULL;
    wcZbrPreviewClass.lpszMenuName  = NULL;
    wcZbrPreviewClass.hbrBackground = GetStockObject(WHITE_BRUSH);
    wcZbrPreviewClass.style         = CS_HREDRAW | CS_VREDRAW;
    wcZbrPreviewClass.cbClsExtra    = 0;
    wcZbrPreviewClass.cbWndExtra    = 0;

    /* Register the class */
    RegisterClass(&wcZbrPreviewClass);

    hInst = hInstance;
    (void) hPrevInstance;

    /* Create applications main window */
    hWndMain = CreateWindow(
      lpszCmdLine, 
      lpszCmdLine, 
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT,       
      CW_USEDEFAULT,       
      CW_USEDEFAULT,       
      CW_USEDEFAULT,       
      NULL,                
      NULL,                
      hInstance,           
      NULL                 
      );

    if (hWndMain == NULL)
        return(FALSE);

    /* Update display of main window */
    ShowWindow(hWndMain, cmdShow);
    UpdateWindow(hWndMain);

    /* Get and dispatch messages for this applicaton */
    while (GetMessage(&msg, NULL, NULL, NULL))
    {
	    TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return(msg.wParam);
}


LRESULT CALLBACK _export ZbrPreviewWndProc(HWND hWnd, UINT message,
  WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_PAINT:
            DisplayZbrFilePreview();
            break;

        case WM_QUERYENDSESSION:
            return((long) TRUE);

        case WM_CLOSE:
            DestroyWindow(hWnd);
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:
            return(DefWindowProc(hWnd, message, wParam, lParam));
    }
    return(0L);
}

/*
** Read and display the preview data stored in a ZBR file.
*/
BOOL DisplayZbrFilePreview(void)
{
    /* Read the header of the ZBR file */
    if (ZBRReadHeader(lpszFileName) == 0)
    {
        GetClientRect(hWndMain, &Rect);

        /* Get the display context */
        hDC = GetDC(hWndMain);

        /* Read and display the preview DIB of the ZBR file */
        ZBRDrawPreview(hDC, Rect);

        /* Release the display context */
        ReleaseDC(hWndMain, hDC);

        return(TRUE);
    }
    else
        return(FALSE);
}

/*
** Read the ZBR file header and preview data.
*/
int ZBRReadHeader(LPSTR FileName)
{
    WORD  Signature;
    WORD  Version;
    HFILE hZbrFile;

    hZbrFile = _lopen(FileName, READ);

    if (hZbrFile == HFILE_ERROR)
        return -1;

    if ((_lread(hZbrFile, &Signature, 2) == 2) && (Signature == ZBRFILEID))
    {
        if ((_lread(hZbrFile, &Version, 2) == 2) && (Version <= ZBRFILEVER))
        {
            if ((_lread(hZbrFile, CommentData, SIZEOFCOMMENT) == SIZEOFCOMMENT))
            {
                if (_lread(hZbrFile, PreviewData, SIZEOFPREVIEW) == SIZEOFPREVIEW)
                {
                    _lclose(hZbrFile);
                    return 0;
                }
            }
        }
    }
    _lclose(hZbrFile);

    return -1;  /* Failed to read correct info, or info correctly */
}

/*
** Display the ZBR preview data.
*/
int ZBRDrawPreview(HDC dc, RECT dest)
{
    int RetVal;
    int nColors;
    int colorAlloc;
    long bitsAlloc;
    HGLOBAL Handle;
    BITMAPINFOHEADER InfoHeader;

    InfoHeader.biSize          = sizeof(BITMAPINFOHEADER);
    InfoHeader.biWidth         = 100;     /* 100 pixels wide */
    InfoHeader.biHeight        = 100;     /* 100 pixels high */
    InfoHeader.biBitCount      = 4;       /* 4 bits per pixel per plane */
    InfoHeader.biPlanes        = 1;       /* One plane */
    InfoHeader.biXPelsPerMeter = 0;       /* Figure out the resolution */
    InfoHeader.biYPelsPerMeter = 0;       /* Ditto */
    InfoHeader.biClrUsed       = 16;      /* 16 colors */
    InfoHeader.biClrImportant  = 0;       /* All colors important */
    InfoHeader.biCompression   = BI_RGB;  /* 4-bit RLE compression possible */
    InfoHeader.biSizeImage     = SIZEOFPREVIEW - 64;  /* Size of preview minus leading 16-color palette */

    nColors = 16;
    colorAlloc = nColors * sizeof(RGBQUAD);
    bitsAlloc = InfoHeader.biSize + colorAlloc + InfoHeader.biSizeImage;
    Handle = GlobalAlloc(GMEM_MOVEABLE, bitsAlloc);

    if (Handle)
    {
        BITMAPINFO *Info = (BITMAPINFO*) GlobalLock(Handle);
        if (Info)
        {
            Info->bmiHeader = InfoHeader;
            hmemcpy(&Info->bmiColors[0], PreviewData, SIZEOFPREVIEW);
            RetVal = StretchDIBits(dc, dest.left, dest.top, dest.right-dest.left,
              dest.bottom-dest.top, 0, 0, 100, 100, PreviewData+64, Info, DIB_RGB_COLORS, SRCCOPY);
            GlobalUnlock(Handle);
            return RetVal;
        }
    }
    return 0;
}
